#include <ctpublic.h>

static char  software_version[]   = "$Id: ct.c,v 1.4 1998/11/13 02:09:10 camber Exp $";
static void *no_unused_var_warn[] = {software_version,
                                     no_unused_var_warn};


CS_RETCODE ct_exit(CS_CONTEXT *ctx, CS_INT unused)
{
	return CS_SUCCEED;
}
CS_RETCODE ct_init(CS_CONTEXT *ctx, CS_INT version)
{
	return CS_SUCCEED;
}
CS_RETCODE ct_con_alloc(CS_CONTEXT *ctx, CS_CONNECTION **con)
{
	*con = (CS_CONNECTION *) malloc(sizeof(CS_CONNECTION));
	(*con)->tds_login = (void *) tds_alloc_login();

	/* so we know who we belong to */
	(*con)->ctx = ctx;

	/* set default values */
	tds_set_library((*con)->tds_login, "CT-Library");
	tds_set_charset((*con)->tds_login, "iso_1");
	tds_set_packet((*con)->tds_login, 512);
	return CS_SUCCEED;
}
CS_RETCODE ct_callback()
{
	return CS_SUCCEED;
}
CS_RETCODE ct_con_props(CS_CONNECTION *con, CS_INT action, CS_INT property,
CS_VOID *buffer, CS_INT buflen, CS_INT *out_len)
{
	/* FIX ME, this will break if buflen != CS_NULLTERM */
	if (action==CS_SET) {
		switch (property) {
			case CS_USERNAME:
				tds_set_user(con->tds_login,(char *)buffer);
				break;
			case CS_PASSWORD:
				tds_set_passwd(con->tds_login,(char *)buffer);
				break;
			default:
				fprintf(stderr,"Unknown property %d\n",property);
		}
	} /* else if */
	return CS_SUCCEED;
}
CS_RETCODE ct_connect(CS_CONNECTION *con, CS_CHAR *servername, CS_INT snamelen)
{
char *server;
int needfree=0;

	if (snamelen==0 || snamelen==CS_UNUSED) {
		server=NULL;
	} else if (snamelen==CS_NULLTERM) {
		server=(char *)servername;
	} else {
		server = (char *) malloc(snamelen+1);
		needfree++;
		strncpy(server,servername,snamelen);
		server[snamelen]='\0';
	}
        tds_set_server(con->tds_login,server);
        con->tds_socket = (void *) tds_connect(con->tds_login);

        tds_set_parent( con->tds_socket, con);

	if (needfree) free(server);

	return CS_SUCCEED;
}
CS_RETCODE ct_cmd_alloc(CS_CONNECTION *con, CS_COMMAND **cmd)
{

	*cmd = (CS_COMMAND *) malloc(sizeof(CS_COMMAND));
	memset(*cmd,'\0',sizeof(CS_COMMAND));

	/* so we know who we belong to */
	(*cmd)->con = con;

	return CS_SUCCEED;
}
CS_RETCODE ct_command(CS_COMMAND *cmd, CS_INT type, CS_VOID *buffer, CS_INT buflen, CS_INT option)
{
	/* FIX ME -- will only work for CS_NULLTERM'd strings */
	if (cmd->query) free(cmd->query);
	cmd->query = (char *) malloc(strlen(buffer));
	strcpy(cmd->query,(char *)buffer);
	return CS_SUCCEED;
}
CS_RETCODE ct_send(CS_COMMAND *cmd)
{
	tds_submit_query(cmd->con->tds_socket, cmd->query);
	return CS_SUCCEED;
}
CS_RETCODE ct_results(CS_COMMAND *cmd, CS_INT *result_type)
{
	switch (tds_process_result_tokens(cmd->con->tds_socket)) {
		case TDS_SUCCEED:
			if (cmd->con->tds_socket->res_info->rows_exist)
				*result_type = CS_ROW_RESULT;
			else
				*result_type = CS_CMD_SUCCEED;
			return CS_SUCCEED;
		case TDS_NO_MORE_RESULTS:
			return CS_END_RESULTS;
		default:
			return CS_FAIL;
	}	
	return CS_FAIL;
}
CS_RETCODE ct_bind(CS_COMMAND *cmd, CS_INT item, CS_DATAFMT *datafmt, CS_VOID *buffer, CS_INT *copied, CS_SMALLINT *indicator)
{
TDSCOLINFO * colinfo;
TDSRESULTINFO * resinfo;
TDSSOCKET * tds;

   tds = (TDSSOCKET *) cmd->con->tds_socket;
   resinfo = tds->res_info;
   colinfo = resinfo->columns[item-1];
   colinfo->varaddr = (char *)buffer;
   colinfo->column_bindtype = datafmt->datatype;
   colinfo->column_bindlen = datafmt->maxlength;

	return CS_SUCCEED;
}
CS_RETCODE ct_fetch(CS_COMMAND *cmd, CS_INT type, CS_INT offset, CS_INT option, CS_INT *rows_read)
{
	switch (tds_process_row_tokens(cmd->con->tds_socket)) {
		case TDS_SUCCEED:
			*rows_read = 1;
			_ct_bind_data(cmd);
			return TDS_SUCCEED;
		case TDS_NO_MORE_ROWS:
			*rows_read = 1;
			_ct_bind_data(cmd);
			return CS_END_DATA;
		default:
			*rows_read = 0;
			return CS_FAIL;
	}
	return CS_SUCCEED;
}
_ct_bind_data(CS_COMMAND *cmd)
{
int i;
TDSCOLINFO *curcol;
TDSRESULTINFO *resinfo = cmd->con->tds_socket->res_info;

   for (i=0;i<resinfo->num_cols;i++)
   {
      curcol = resinfo->columns[i];
		/* currently there is no NULL handling for ctlib, this is where it 
			will be */
      /* printf("%d %s\n",i,resinfo->columns[i]->column_value); */
      if (curcol->varaddr)
      {
			/* FIX ME -- I only work with strings so far */
      	strcpy(curcol->varaddr, &resinfo->current_row[curcol->column_offset]);
	/* curcol->column_value); */
			/* FIX ME -- convert functions need to be commonized between libs 
				that shouldn't be very hard since we used server types for all the 
				convert functions.
			*/
         /* desttype = _ct_get_server_type(curcol->column_bindtype);
         srctype = _get_conversion_type(curcol->column_type,curcol->column_size);
         dbconvert(dbproc,srctype,
            curcol->column_value,-1,desttype,
            curcol->varaddr,curcol->column_bindlen); */
		}
	}
}
CS_RETCODE ct_cmd_drop(CS_COMMAND *cmd)
{
	if (cmd) free(cmd);
	return CS_SUCCEED;
}
CS_RETCODE ct_close(CS_CONNECTION *con, CS_INT option)
{
	close(con->tds_socket->s);
	return CS_SUCCEED;
}
CS_RETCODE ct_con_drop(CS_CONNECTION *con)
{
	if (con) free(con);
	return CS_SUCCEED;
}
int _ct_get_server_type(int datatype)
{
   switch (datatype) {
      case CS_CHAR_TYPE:
         return SYBCHAR;
         break;
      case CS_INT_TYPE:
         return SYBINT4;
         break;
      case CS_SMALLINT_TYPE:
         return SYBINT2;
         break;
      case CS_TINYINT_TYPE:
         return SYBINT1;
         break;
   }
}
CS_RETCODE ct_cancel(CS_CONNECTION *conn, CS_COMMAND *cmd, CS_INT type)
{
}
CS_RETCODE ct_describe(CS_COMMAND *cmd, CS_INT item, CS_DATAFMT *datafmt)
{
}
CS_RETCODE ct_res_info(CS_COMMAND *cmd, CS_INT type, CS_VOID *buffer, CS_INT buflen, CS_INT *out_len)
{
}
